@@ -30,46 +30,60 @@ import (
 	"github.com/spf13/cobra"
 )
 
+var nfsPermPreserveXfers = map[common.FromTo]bool{
+	common.EFromTo.LocalFileNFS():   true,
+	common.EFromTo.FileNFSLocal():   true,
+	common.EFromTo.FileNFSFileNFS(): true,
+	common.EFromTo.FileNFSFileSMB(): true,
+	common.EFromTo.FileSMBFileNFS(): true,
+}
+
 func validatePreserveNFSPropertyOption(toPreserve bool, fromTo common.FromTo, flagName string) error {
 	// preserverInfo will be true by default for NFS-aware locations unless specified false.
-	// 1. Upload (Windows/Linux -> Azure File)
-	// 2. Download (Azure File -> Windows/Linux)
+	// 1. Upload (Linux -> Azure File)
+	// 2. Download (Azure File -> Linux)
 	// 3. S2S (Azure File -> Azure File)
-	// TODO: More combination checks to be added later
-	if toPreserve && !(fromTo == common.EFromTo.LocalFileNFS() ||
-		fromTo == common.EFromTo.FileNFSLocal() ||
-		fromTo == common.EFromTo.FileNFSFileNFS()) {
-		return fmt.Errorf("%s is set but the job is not between %s-aware resources", flagName, common.Iff(flagName == PreserveInfoFlag, "permission", "NFS"))
-	}
 
-	if toPreserve && (fromTo.IsUpload() || fromTo.IsDownload()) &&
-		runtime.GOOS != "windows" && runtime.GOOS != "linux" {
-		return fmt.Errorf("%s is set but persistence for up/downloads is supported only in Windows and Linux", flagName)
+	if toPreserve {
+		// The user cannot preserve permissions between SMB->NFS or NFS->SMB transfers.
+		if flagName == PreservePermissionsFlag && (fromTo == common.EFromTo.FileNFSFileSMB() || fromTo == common.EFromTo.FileSMBFileNFS()) {
+			return fmt.Errorf("--preserve-permissions flag is not supported for cross-protocol transfers (i,e. SMB->NFS, NFS->SMB). Please remove this flag and try again.")
+		} else if !nfsPermPreserveXfers[fromTo] {
+			return fmt.Errorf("%s is set but the job is not between %s-aware resources", flagName, common.Iff(flagName == PreserveInfoFlag, "permission", "NFS"))
+		} else if (fromTo.IsUpload() || fromTo.IsDownload()) && runtime.GOOS != "linux" {
+			return fmt.Errorf("%s is set but persistence for up/downloads is supported only in Linux", flagName)
+		}
 	}
-
 	return nil
 }
 
+var smbPermPreserveXfers = map[common.FromTo]bool{
+	common.EFromTo.LocalFile():      true,
+	common.EFromTo.FileLocal():      true,
+	common.EFromTo.FileFile():       true,
+	common.EFromTo.LocalFileSMB():   true,
+	common.EFromTo.FileSMBLocal():   true,
+	common.EFromTo.FileSMBFileSMB(): true,
+}
+
 func validatePreserveSMBPropertyOption(toPreserve bool, fromTo common.FromTo, flagName string) error {
 	// preserverInfo will be true by default for SMB-aware locations unless specified false.
 	// 1. Upload (Windows/Linux -> Azure File)
 	// 2. Download (Azure File -> Windows/Linux)
 	// 3. S2S (Azure File -> Azure File)
-	if toPreserve && flagName == PreservePermissionsFlag &&
-		(fromTo == common.EFromTo.BlobBlob() || fromTo == common.EFromTo.BlobFSBlob() || fromTo == common.EFromTo.BlobBlobFS() || fromTo == common.EFromTo.BlobFSBlobFS()) {
-		// the user probably knows what they're doing if they're trying to persist permissions between blob-type endpoints.
-		return nil
-	} else if toPreserve && !(fromTo == common.EFromTo.LocalFile() ||
-		fromTo == common.EFromTo.FileLocal() ||
-		fromTo == common.EFromTo.FileFile()) {
-		return fmt.Errorf("%s is set but the job is not between %s-aware resources", flagName, common.Iff(flagName == PreservePermissionsFlag, "permission", "SMB"))
+	if toPreserve {
+		if flagName == PreservePermissionsFlag &&
+			(fromTo == common.EFromTo.BlobBlob() || fromTo == common.EFromTo.BlobFSBlob() ||
+				fromTo == common.EFromTo.BlobBlobFS() || fromTo == common.EFromTo.BlobFSBlobFS()) {
+			// the user probably knows what they're doing if they're trying to persist permissions between blob-type endpoints.
+			return nil
+		} else if !smbPermPreserveXfers[fromTo] {
+			return fmt.Errorf("%s is set but the job is not between %s-aware resources", flagName, common.Iff(flagName == PreservePermissionsFlag, "permission", "SMB"))
+		} else if (fromTo.IsUpload() || fromTo.IsDownload()) &&
+			runtime.GOOS != "windows" && runtime.GOOS != "linux" {
+			return fmt.Errorf("%s is set but persistence for up/downloads is supported only in Windows and Linux", flagName)
+		}
 	}
-
-	if toPreserve && (fromTo.IsUpload() || fromTo.IsDownload()) &&
-		runtime.GOOS != "windows" && runtime.GOOS != "linux" {
-		return fmt.Errorf("%s is set but persistence for up/downloads is supported only in Windows and Linux", flagName)
-	}
-
 	return nil
 }
 
@@ -77,12 +91,17 @@ func areBothLocationsNFSAware(fromTo common.FromTo) bool {
 	// 1. Upload (Linux -> Azure File)
 	// 2. Download (Azure File -> Linux)
 	// 3. S2S (Azure File -> Azure File) (Works on Windows,Linux,Mac)
+
+	var s2sNFSXfers = map[common.FromTo]bool{
+		common.EFromTo.FileNFSFileNFS(): true,
+		common.EFromTo.FileNFSFileSMB(): true,
+		common.EFromTo.FileSMBFileNFS(): true,
+	}
+
 	if (runtime.GOOS == "linux") &&
 		(fromTo == common.EFromTo.LocalFileNFS() || fromTo == common.EFromTo.FileNFSLocal()) {
-		common.SetNFSFlag(true)
 		return true
-	} else if fromTo == common.EFromTo.FileNFSFileNFS() {
-		common.SetNFSFlag(true)
+	} else if s2sNFSXfers[fromTo] {
 		return true
 	} else {
 		return false
@@ -133,28 +152,14 @@ func GetPreserveInfoFlagDefault(cmd *cobra.Command, fromTo common.FromTo) bool {
 func performNFSSpecificValidation(fromTo common.FromTo,
 	preservePermissions common.PreservePermissionsOption,
 	preserveInfo bool,
-	symlinkHandling common.SymlinkHandlingType,
-	hardlinkHandling common.HardlinkHandlingType) (err error) {
+	hardlinkHandling *common.HardlinkHandlingType,
+	symlinkHandling common.SymlinkHandlingType) (err error) {
 
 	// check for unsupported NFS behavior
 	if isUnsupported, err := isUnsupportedPlatformForNFS(fromTo); isUnsupported {
 		return err
 	}
 
-	// If we are not preserving original file permissions (raw.preservePermissions == false),
-	// and the operation is a file copy from azure file NFS to local linux (FromTo == FileLocal),
-	// and the current OS is Linux, then we require root privileges to proceed.
-	//
-	// This is because modifying file ownership or permissions on Linux
-	// typically requires elevated privileges. To safely handle permission
-	// changes during the local file operation, we enforce that the process
-	// must be running as root.
-	if !preservePermissions.IsTruthy() && fromTo == common.EFromTo.FileNFSLocal() {
-		if err := common.EnsureRunningAsRoot(); err != nil {
-			return fmt.Errorf("failed to copy source to destination without preserving permissions: operation not permitted. Please retry with root privileges or use the default option (--preserve-permissions=true)")
-		}
-	}
-
 	if err = validatePreserveNFSPropertyOption(preserveInfo,
 		fromTo,
 		PreserveInfoFlag); err != nil {
@@ -165,12 +170,13 @@ func performNFSSpecificValidation(fromTo common.FromTo,
 		PreservePermissionsFlag); err != nil {
 		return err
 	}
+	// TODO: Add this check in Phase-3 which targets to support hardlinks for NFS copy.
+	// if err = validateAndAdjustHardlinksFlag(hardlinkHandling, fromTo); err != nil {
+	// 	return err
+	// }
 
-	if err = validateSymlinkFlag(symlinkHandling == common.ESymlinkHandlingType.Follow(), symlinkHandling == common.ESymlinkHandlingType.Preserve()); err != nil {
-		return err
-	}
-
-	if err = validateHardlinksFlag(hardlinkHandling, fromTo); err != nil {
+	if err = validateSymlinkFlag(symlinkHandling == common.ESymlinkHandlingType.Follow(),
+		fromTo); err != nil {
 		return err
 	}
 	return nil
@@ -192,8 +198,7 @@ func performNFSSpecificValidation(fromTo common.FromTo,
 func performSMBSpecificValidation(fromTo common.FromTo,
 	preservePermissions common.PreservePermissionsOption,
 	preserveInfo bool,
-	preservePOSIXProperties bool,
-	hardlinkHandling common.HardlinkHandlingType) (err error) {
+	preservePOSIXProperties bool) (err error) {
 
 	if err = validatePreserveSMBPropertyOption(preserveInfo,
 		fromTo,
@@ -208,47 +213,84 @@ func performSMBSpecificValidation(fromTo common.FromTo,
 		PreservePermissionsFlag); err != nil {
 		return err
 	}
-	if err = validateHardlinksFlag(hardlinkHandling, fromTo); err != nil {
-		return err
-	}
+
+	// TODO: Add this check in Phase-3 which targets to support hardlinks for NFS copy.
+	// if err = validateAndAdjustHardlinksFlag(hardlinkHandling, fromTo); err != nil {
+	// 	return err
+	// }
 	return nil
 }
 
-// validateSymlinkFlag checks whether the '--follow-symlink' or '--preserve-symlink' flags
-// are set for an NFS copy operation. Since symlink support is not available for NFS,
-// the function returns an error if either flag is enabled.
-// By default, symlink files will be skipped during NFS copy.
-func validateSymlinkFlag(followSymlinks, preserveSymlinks bool) error {
+// validateSymlinkFlag checks if the --follow-symlink flag is valid for uploading from local filesystem.
+func validateSymlinkFlag(followSymlinks bool, fromTo common.FromTo) error {
 
 	if followSymlinks {
-		return fmt.Errorf("The '--follow-symlink' flag is not supported for NFS copy. Symlink files will be skipped by default.")
-
-	}
-	if preserveSymlinks {
-		return fmt.Errorf("the --preserve-symlink flag is not support for NFS copy. Symlink files will be skipped by default.")
+		if fromTo.From() != common.ELocation.Local() {
+			return fmt.Errorf("The '--follow-symlink' flag is only applicable when uploading from local filesystem.")
+		}
 	}
 	return nil
 }
 
-func validateHardlinksFlag(option common.HardlinkHandlingType, fromTo common.FromTo) error {
+// validateAndAdjustHardlinksFlag validates and adjusts the --hardlinks option based on OS,
+// transfer direction (upload, download, S2S), and source/destination types (NFS, SMB, local).
+// Returns an error if the configuration is unsupported.
+// This function will be added as part of Phase-3 which targets to support hardlinks for NFS copy.
+// func validateAndAdjustHardlinksFlag(option *common.HardlinkHandlingType, fromTo common.FromTo) error {
+// 	if !fromTo.IsNFS() {
+// 		return nil
+// 	}
 
-	// Validate for Download: Only allowed when downloading from an NFS share to a Linux filesystem
-	if common.IsNFSCopy() {
-		if runtime.GOOS == "linux" && fromTo.IsDownload() && (fromTo.From() != common.ELocation.FileNFS()) {
-			return fmt.Errorf("The --hardlinks option, when downloading, is only supported from a NFS file share to a Linux filesystem.")
-		}
+// 	// NFS<->SMB special case: force skip
+// 	if (fromTo == common.EFromTo.FileNFSFileSMB() || fromTo == common.EFromTo.FileSMBFileNFS()) &&
+// 		*option != common.SkipHardlinkHandlingType {
+// 		return fmt.Errorf(
+// 			"For NFS->SMB and SMB->NFS transfers, '--hardlinks' must be set to 'skip'. " +
+// 				"Hardlinked files are not supported between NFS and SMB and will always be skipped. " +
+// 				"Please re-run with '--hardlinks=skip'.",
+// 		)
+// 	}
 
-		// Validate for Upload or S2S: Only allowed when uploading *to* a local file system
-		if runtime.GOOS == "linux" && (fromTo.IsUpload() || fromTo.IsS2S()) && (fromTo.To() != common.ELocation.FileNFS()) {
-			return fmt.Errorf("The --hardlinks option, when uploading, is only supported from a NFS file share to a Linux filesystem or between NFS file shares.")
-		}
-	}
+// 	// OS check: hardlinks handling only supported on Linux in case of upload and download
+// 	if runtime.GOOS != "linux" && !fromTo.IsS2S() {
+// 		return fmt.Errorf("The --hardlinks option is only supported on Linux.")
+// 	}
 
-	if common.IsNFSCopy() && option == common.DefaultHardlinkHandlingType {
-		glcm.Info("The --hardlinks option is set to 'follow'. Hardlinked files will be copied as a regular file at the destination.")
-	}
-	return nil
-}
+// 	switch {
+// 	case fromTo.IsDownload():
+// 		// Must be NFS -> Local Linux
+// 		if fromTo.From() != common.ELocation.FileNFS() {
+// 			return fmt.Errorf("For downloads, '--hardlinks' is only supported from an NFS file share to a Linux filesystem.")
+// 		}
+
+// 	case fromTo.IsUpload():
+// 		// Must be Local Linux -> NFS
+// 		if fromTo.To() != common.ELocation.FileNFS() {
+// 			return fmt.Errorf("For uploads, '--hardlinks' is only supported from a Linux filesystem to an NFS file share.")
+// 		}
+
+// 	case fromTo.IsS2S():
+// 		// Allowed: NFS<->NFS, NFS->SMB, SMB->NFS
+// 		validPairs := map[common.FromTo]bool{
+// 			common.EFromTo.FileNFSFileNFS(): true,
+// 			common.EFromTo.FileNFSFileSMB(): true,
+// 			common.EFromTo.FileSMBFileNFS(): true,
+// 		}
+// 		if !validPairs[fromTo] {
+// 			return fmt.Errorf("For S2S transfers, '--hardlinks' is only supported for NFS<->NFS, NFS->SMB, and SMB->NFS.")
+// 		}
+// 	}
+
+// 	// Info messages
+// 	switch *option {
+// 	case common.SkipHardlinkHandlingType:
+// 		glcm.Info("The --hardlinks option is set to 'skip'. Hardlinked files will be skipped.")
+// 	case common.DefaultHardlinkHandlingType:
+// 		glcm.Info("The --hardlinks option is set to 'follow'. Hardlinked files will be copied as regular files.")
+// 	}
+
+// 	return nil
+// }
 
 func isUnsupportedPlatformForNFS(fromTo common.FromTo) (bool, error) {
 	// upload and download is not supported for NFS on non-linux systems
@@ -273,28 +315,34 @@ func validateShareProtocolCompatibility(
 	resource common.ResourceString,
 	serviceClient *common.ServiceClient,
 	isSource bool,
-	protocol string,
+	protocol common.Location,
+	fromTo common.FromTo,
 ) error {
-	if protocol == "" {
-		return nil
-	}
-
-	direction := "from"
-	if !isSource {
-		direction = "to"
-	}
 
 	// We can ignore the error if we fail to get the share properties.
 	shareProtocol, _ := getShareProtocolType(ctx, serviceClient, resource, protocol)
 
-	if shareProtocol == "SMB" && common.IsNFSCopy() {
-		return fmt.Errorf("The %s share has SMB protocol enabled. To copy %s a SMB share, use the appropriate --from-to flag value", direction, direction)
+	if shareProtocol == common.ELocation.File() {
+		if isSource && fromTo.From() != common.ELocation.File() {
+			return errors.New("the source share has SMB protocol enabled. " +
+				"To copy from a SMB share, use the appropriate --from-to flag value")
+		}
+		if !isSource && fromTo.To() != common.ELocation.File() {
+			return errors.New("the destination share has NFS protocol enabled. " +
+				"To copy to a NFS share, use the appropriate --from-to flag value")
+		}
 	}
 
-	if shareProtocol == "NFS" && !common.IsNFSCopy() {
-		return fmt.Errorf("The %s share has NFS protocol enabled. To copy %s a NFS share, use the appropriate --from-to flag value", direction, direction)
+	if shareProtocol == common.ELocation.FileNFS() {
+		if isSource && fromTo.From() != common.ELocation.FileNFS() {
+			return errors.New("the source share has NFS protocol enabled. " +
+				"To copy from a NFS share, use the appropriate --from-to flag value")
+		}
+		if !isSource && fromTo.To() != common.ELocation.FileNFS() {
+			return errors.New("the destination share has NFS protocol enabled. " +
+				"To copy to a NFS share, use the appropriate --from-to flag value")
+		}
 	}
-
 	return nil
 }
 
@@ -304,18 +352,18 @@ func getShareProtocolType(
 	ctx context.Context,
 	serviceClient *common.ServiceClient,
 	resource common.ResourceString,
-	givenValue string,
-) (string, error) {
+	givenValue common.Location,
+) (common.Location, error) {
 
 	fileURLParts, err := file.ParseURL(resource.Value)
 	if err != nil {
-		return "UNKNOWN", fmt.Errorf("failed to parse resource URL: %w", err)
+		return common.ELocation.Unknown(), fmt.Errorf("failed to parse resource URL: %w", err)
 	}
 	shareName := fileURLParts.ShareName
 
 	fileServiceClient, err := serviceClient.FileServiceClient()
 	if err != nil {
-		return "UNKNOWN", fmt.Errorf("failed to create file service client: %w", err)
+		return common.ELocation.Unknown(), fmt.Errorf("failed to create file service client: %w", err)
 	}
 
 	shareClient := fileServiceClient.NewShareClient(shareName)
@@ -325,59 +373,59 @@ func getShareProtocolType(
 		return givenValue, err
 	}
 
-	if properties.EnabledProtocols == nil {
-		return "SMB", nil // Default assumption
+	if properties.EnabledProtocols == nil || *properties.EnabledProtocols == "SMB" {
+		return common.ELocation.File(), nil // Default assumption
 	}
 
-	return *properties.EnabledProtocols, nil
+	return common.ELocation.FileNFS(), nil
 }
 
 // Protocol compatibility validation for SMB and NFS transfers
 func validateProtocolCompatibility(ctx context.Context, fromTo common.FromTo, src, dst common.ResourceString, srcClient, dstClient *common.ServiceClient) error {
 
-	getUploadDownloadProtocol := func(fromTo common.FromTo) string {
+	getUploadDownloadProtocol := func(fromTo common.FromTo) common.Location {
 		switch fromTo {
 		case common.EFromTo.LocalFile(), common.EFromTo.FileLocal():
-			return "SMB"
+			return common.ELocation.File()
 		case common.EFromTo.LocalFileNFS(), common.EFromTo.FileNFSLocal():
-			return "NFS"
+			return common.ELocation.FileNFS()
 		default:
-			return ""
+			return common.ELocation.Unknown()
 		}
 	}
 
-	var protocol string
+	var srcProtocol, dstProtocol common.Location
 
 	// S2S Transfers
 	if fromTo.IsS2S() {
 		switch fromTo {
 		case common.EFromTo.FileFile():
-			protocol = "SMB"
+			srcProtocol, dstProtocol = common.ELocation.File(), common.ELocation.File()
 		case common.EFromTo.FileNFSFileNFS():
-			protocol = "NFS"
-		default:
-			if common.IsNFSCopy() {
-				return errors.New("NFS copy is not supported for cross-protocol transfers, i.e., Files SMB to Files NFS or vice versa")
-			}
+			srcProtocol, dstProtocol = common.ELocation.FileNFS(), common.ELocation.FileNFS()
+		case common.EFromTo.FileNFSFileSMB():
+			srcProtocol, dstProtocol = common.ELocation.FileNFS(), common.ELocation.File()
+		case common.EFromTo.FileSMBFileNFS():
+			srcProtocol, dstProtocol = common.ELocation.File(), common.ELocation.FileNFS()
 		}
 
 		// Validate both source and destination
-		if err := validateShareProtocolCompatibility(ctx, src, srcClient, true, protocol); err != nil {
+		if err := validateShareProtocolCompatibility(ctx, src, srcClient, true, srcProtocol, fromTo); err != nil {
 			return err
 		}
-		return validateShareProtocolCompatibility(ctx, dst, dstClient, false, protocol)
+		return validateShareProtocolCompatibility(ctx, dst, dstClient, false, dstProtocol, fromTo)
 	}
 
 	// Uploads to File Shares
 	if fromTo.IsUpload() {
-		protocol = getUploadDownloadProtocol(fromTo)
-		return validateShareProtocolCompatibility(ctx, dst, dstClient, false, protocol)
+		dstProtocol = getUploadDownloadProtocol(fromTo)
+		return validateShareProtocolCompatibility(ctx, dst, dstClient, false, dstProtocol, fromTo)
 	}
 
 	// Downloads from File Shares
 	if fromTo.IsDownload() {
-		protocol = getUploadDownloadProtocol(fromTo)
-		return validateShareProtocolCompatibility(ctx, src, srcClient, true, protocol)
+		srcProtocol = getUploadDownloadProtocol(fromTo)
+		return validateShareProtocolCompatibility(ctx, src, srcClient, true, srcProtocol, fromTo)
 	}
 
 	return nil
@@ -401,11 +449,11 @@ func ComputePreserveFlags(cmd *cobra.Command, userFromTo common.FromTo, preserve
 
 	// Final preservePermissions logic
 	finalPreservePermissions := preservePermissions
-	if !common.IsNFSCopy() {
+	if !userFromTo.IsNFS() {
 		finalPreservePermissions = preservePermissions || preserveSMBPermissions
 	}
 
-	if common.IsNFSCopy() && ((preserveSMBInfo && runtime.GOOS == "linux") || preserveSMBPermissions) {
+	if userFromTo.IsNFS() && ((preserveSMBInfo && runtime.GOOS == "linux") || preserveSMBPermissions) {
 		glcm.Error(InvalidFlagsForNFSMsg)
 	}
 
