@@ -54,7 +54,10 @@ func ToFixed(num float64, precision int) float64 {
 // MainSTE initializes the Storage Transfer Engine
 func MainSTE(concurrency ste.ConcurrencySettings, targetRateInMegaBitsPerSec float64) error {
 	// Initialize the JobsAdmin, resurrect Job plan files
-	initJobsAdmin(steCtx, concurrency, targetRateInMegaBitsPerSec)
+	err := initJobsAdmin(steCtx, concurrency, targetRateInMegaBitsPerSec)
+	if err != nil {
+		return err
+	}
 	// TODO: We may want to list listen first and terminate if there is already an instance listening
 
 	// if we've a custom mime map
@@ -82,8 +85,8 @@ var ExecuteNewCopyJobPartOrder =
 func(order common.CopyJobPartOrderRequest) common.CopyJobPartOrderResponse {
 	// Get the file name for this Job Part's Plan
 	jppfn := JobsAdmin.NewJobPartPlanFileName(order.JobID, order.PartNum)
-	jppfn.Create(order)                                                                  // Convert the order to a plan file
-	jm := JobsAdmin.JobMgrEnsureExists(order.JobID, order.LogLevel, order.CommandString) // Get a this job part's job manager (create it if it doesn't exist)
+	jppfn.Create(order)                                                                                         // Convert the order to a plan file
+	jm := JobsAdmin.JobMgrEnsureExists(order.JobID, order.LogLevel, order.CommandString, order.JobErrorHandler) // Get a this job part's job manager (create it if it doesn't exist)
 
 	if len(order.Transfers.List) == 0 && order.IsFinalPart {
 		/*
@@ -93,12 +96,6 @@ func(order common.CopyJobPartOrderRequest) common.CopyJobPartOrderResponse {
 		 */
 		jm.Log(common.LogWarning, "No transfers were scheduled.")
 	}
-	// Get credential info from RPC request order, and set in InMemoryTransitJobState.
-	jm.SetInMemoryTransitJobState(
-		ste.InMemoryTransitJobState{
-			CredentialInfo:          order.CredentialInfo,
-			S2SSourceCredentialType: order.S2SSourceCredentialType,
-		})
 	// Supply no plan MMF because we don't have one, and AddJobPart will create one on its own.
 	// Add this part to the Job and schedule its transfers
 
@@ -118,7 +115,6 @@ func(order common.CopyJobPartOrderRequest) common.CopyJobPartOrderResponse {
 		ScheduleTransfers: true,
 	}
 	jm.AddJobPart(args)
-
 	// Update jobPart Status with the status Manager
 	jm.SendJobPartCreatedMsg(ste.JobPartCreatedMsg{TotalTransfers: uint32(len(order.Transfers.List)),
 		IsFinalPart:             order.IsFinalPart,
@@ -126,7 +122,8 @@ func(order common.CopyJobPartOrderRequest) common.CopyJobPartOrderResponse {
 		FileTransfers:           order.Transfers.FileTransferCount,
 		SymlinkTransfers:        order.Transfers.SymlinkTransferCount,
 		FolderTransfer:          order.Transfers.FolderTransferCount,
-		HardlinksConvertedCount: order.Transfers.HardlinksConvertedCount})
+		HardlinksConvertedCount: order.Transfers.HardlinksConvertedCount,
+	})
 
 	return common.CopyJobPartOrderResponse{JobStarted: true}
 }
@@ -138,12 +135,12 @@ func(order common.CopyJobPartOrderRequest) common.CopyJobPartOrderResponse {
     * If a job is already paused, it cannot be paused again
 */
 
-func CancelPauseJobOrder(jobID common.JobID, desiredJobStatus common.JobStatus) common.CancelPauseResumeResponse {
+func CancelPauseJobOrder(jobID common.JobID, desiredJobStatus common.JobStatus, jobErrorHandler common.JobErrorHandler) common.CancelPauseResumeResponse {
 	jm, found := JobsAdmin.JobMgr(jobID) // Find Job being paused/canceled
 	if !found {
 		// If the Job is not found, search for Job Plan files in the existing plan file
 		// and resurrect the job
-		if !JobsAdmin.ResurrectJob(jobID, nil, nil, false) {
+		if !JobsAdmin.ResurrectJob(jobID, nil, nil, false, jobErrorHandler) {
 			return common.CancelPauseResumeResponse{
 				CancelledPauseResumed: false,
 				ErrorMsg:              fmt.Sprintf("no active job with JobId %s exists", jobID.String()),
@@ -164,7 +161,7 @@ func ResumeJobOrder(req common.ResumeJobRequest) common.CancelPauseResumeRespons
 	}
 	// Always search the plan files in Azcopy folder,
 	// and resurrect the Job with provided credentials, to ensure SAS and etc get updated.
-	if !JobsAdmin.ResurrectJob(req.JobID, req.SrcServiceClient, req.DstServiceClient, false) {
+	if !JobsAdmin.ResurrectJob(req.JobID, req.SrcServiceClient, req.DstServiceClient, false, req.JobErrorHandler) {
 		return common.CancelPauseResumeResponse{
 			CancelledPauseResumed: false,
 			ErrorMsg:              fmt.Sprintf("no job with JobId %v exists", req.JobID),
@@ -204,59 +201,6 @@ func ResumeJobOrder(req common.ResumeJobRequest) common.CancelPauseResumeRespons
 		}
 	}
 
-	// If the credential type is is Anonymous, to resume the Job destinationSAS / sourceSAS needs to be provided
-	// Depending on the FromType, sourceSAS or destinationSAS is checked.
-	if req.CredentialInfo.CredentialType == common.ECredentialType.Anonymous() {
-		var errorMsg = ""
-		switch jpm.Plan().FromTo {
-		case common.EFromTo.LocalBlob(),
-			common.EFromTo.LocalFile(),
-			common.EFromTo.LocalFileNFS(),
-			common.EFromTo.S3Blob(),
-			common.EFromTo.GCPBlob():
-			if len(req.DestinationSAS) == 0 {
-				errorMsg = "The destination-sas switch must be provided to resume the job"
-			}
-		case common.EFromTo.BlobLocal(),
-			common.EFromTo.FileLocal(),
-			common.EFromTo.FileNFSLocal(),
-			common.EFromTo.BlobTrash(),
-			common.EFromTo.FileTrash():
-			if len(req.SourceSAS) == 0 {
-				plan := jpm.Plan()
-				if plan.FromTo.From() == common.ELocation.Blob() {
-					src := string(plan.SourceRoot[:plan.SourceRootLength])
-					if common.IsSourcePublicBlob(src, steCtx) {
-						break
-					}
-				}
-
-				errorMsg = "The source-sas switch must be provided to resume the job"
-			}
-		case common.EFromTo.BlobBlob(),
-			common.EFromTo.FileBlob():
-			if len(req.SourceSAS) == 0 ||
-				len(req.DestinationSAS) == 0 {
-
-				plan := jpm.Plan()
-				if plan.FromTo.From() == common.ELocation.Blob() && len(req.DestinationSAS) != 0 {
-					src := string(plan.SourceRoot[:plan.SourceRootLength])
-					if common.IsSourcePublicBlob(src, steCtx) {
-						break
-					}
-				}
-
-				errorMsg = "Both the source-sas and destination-sas switches must be provided to resume the job"
-			}
-		}
-		if len(errorMsg) != 0 {
-			return common.CancelPauseResumeResponse{
-				CancelledPauseResumed: false,
-				ErrorMsg:              fmt.Sprintf("cannot resume job with JobId %s. %s", req.JobID, errorMsg),
-			}
-		}
-	}
-
 	// After creating the Job mgr, set the include / exclude list of transfer.
 	jm.SetIncludeExclude(req.IncludeTransfer, req.ExcludeTransfer)
 	jpp0 := jpm.Plan()
@@ -275,15 +219,11 @@ func ResumeJobOrder(req common.ResumeJobRequest) common.CancelPauseResumeRespons
 		common.EJobStatus.CompletedWithSkipped(),
 		common.EJobStatus.CompletedWithErrorsAndSkipped(),
 		common.EJobStatus.Cancelled(),
-		common.EJobStatus.Paused():
+		common.EJobStatus.Paused(),
+		common.EJobStatus.Failed():
 		// go func() {
 		// Navigate through transfers and schedule them independently
 		// This is done to avoid FE to get blocked until all the transfers have been scheduled
-		// Get credential info from RPC request, and set in InMemoryTransitJobState.
-		jm.SetInMemoryTransitJobState(
-			ste.InMemoryTransitJobState{
-				CredentialInfo: req.CredentialInfo,
-			})
 
 		// Prevents previous number of failed transfers seeping into a new run
 		jm.ResetFailedTransfersCount()
@@ -343,7 +283,7 @@ func GetJobSummary(jobID common.JobID) common.ListJobSummaryResponse {
 		// Job with JobId does not exists
 		// Search the plan files in Azcopy folder
 		// and resurrect the Job
-		if !JobsAdmin.ResurrectJob(jobID, nil, nil, false) {
+		if !JobsAdmin.ResurrectJob(jobID, nil, nil, false, warnJobErrorHandler{jobID: jobID}) {
 			return common.ListJobSummaryResponse{
 				ErrorMsg: fmt.Sprintf("no job with JobId %v exists", jobID),
 			}
@@ -582,7 +522,7 @@ func ListJobTransfers(r common.ListJobTransfersRequest) common.ListJobTransfersR
 		// Job with JobId does not exists
 		// Search the plan files in Azcopy folder
 		// and resurrect the Job
-		if !JobsAdmin.ResurrectJob(r.JobID, nil, nil, false) {
+		if !JobsAdmin.ResurrectJob(r.JobID, nil, nil, false, warnJobErrorHandler{jobID: r.JobID}) {
 			return common.ListJobTransfersResponse{
 				ErrorMsg: fmt.Sprintf("no job with JobId %v exists", r.JobID),
 			}
@@ -630,27 +570,13 @@ func ListJobTransfers(r common.ListJobTransfersRequest) common.ListJobTransfersR
 	return ljt
 }
 
-func GetJobLCMWrapper(jobID common.JobID) common.LifecycleMgr {
-	jobmgr, found := JobsAdmin.JobMgr(jobID)
-	lcm := common.GetLifecycleMgr()
-
-	if !found {
-		return lcm
-	}
-
-	return ste.JobLogLCMWrapper{
-		JobManager:   jobmgr,
-		LifecycleMgr: lcm,
-	}
-}
-
 // GetJobDetails api returns the job FromTo info.
 func GetJobDetails(r common.GetJobDetailsRequest) common.GetJobDetailsResponse {
 	jm, found := JobsAdmin.JobMgr(r.JobID)
 	if !found {
 		// Job with JobId does not exists.
 		// Search the plan files in Azcopy folder and resurrect the Job.
-		if !JobsAdmin.ResurrectJob(r.JobID, nil, nil, false) {
+		if !JobsAdmin.ResurrectJob(r.JobID, nil, nil, false, warnJobErrorHandler{jobID: r.JobID}) {
 			return common.GetJobDetailsResponse{
 				ErrorMsg: fmt.Sprintf("Job with JobID %v does not exist or is invalid", r.JobID),
 			}
@@ -682,3 +608,11 @@ func GetJobDetails(r common.GetJobDetailsRequest) common.GetJobDetailsResponse {
 		TrailingDot: jp0.Plan().DstFileData.TrailingDot,
 	}
 }
+
+type warnJobErrorHandler struct {
+	jobID common.JobID
+}
+
+func (w warnJobErrorHandler) Error(err string) {
+	panic("We don't expect errors to be hit for job " + w.jobID.String() + ". error: " + err)
+}
